﻿using Grayscale.P007_Csv________.L500____Util;
using Grayscale.P021_ConstShogi_.L250____Const;
using Grayscale.P213_Komasyurui_.L250____Word;
using Grayscale.P505_Atama_Unou_.L___250_Unou;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
#if DEBUG
#endif

namespace Grayscale.P505_Atama_Unou_.L250____Unou
{

    /// <summary>
    /// 飽きてくるカウンター。
    /// 
    /// パラメーター構成は、フィーチャー・ベクターと一緒。
    /// 入っている値が、評価値ではなくて、更新回数。
    /// </summary>
    public class ResultVectorImpl : ResultVector
    {

        /// <summary>
        /// 駒割の更新量。
        /// </summary>
        public double[] Komawari { get; set; }

        /// <summary>
        /// 二駒関係ＫＫ☆の更新量
        /// 
        /// 自玉の位置８１マス　×　相手玉の位置８１マス
        /// </summary>
        public double[,] NikomaKankei_KK { get; set; }

        /// <summary>
        /// 二駒関係ＫＰの更新量
        /// 
        /// 自玉の位置８１マス　×　１２２４種類の調査項目
        /// </summary>
        public double[,] NikomaKankei_KP { get; set; }

        /// <summary>
        /// 二駒関係ＰＰ☆の更新量
        /// 
        /// １２２４種類の調査項目　×　１２２４種類の調査項目
        /// </summary>
        public double[,] NikomaKankei_PP { get; set; }

        public ResultVectorImpl()
        {
            this.Komawari = new double[Array_Komasyurui.Items_All.Length];
            this.NikomaKankei_KK = new double[ConstShogi.BAN_SIZE, ConstShogi.BAN_SIZE];
            this.NikomaKankei_KP = new double[ConstShogi.BAN_SIZE, FeatureVectorImpl.CHOSA_KOMOKU_P];
            this.NikomaKankei_PP = new double[FeatureVectorImpl.CHOSA_KOMOKU_P, FeatureVectorImpl.CHOSA_KOMOKU_P];
        }


        /// <summary>
        /// ファイルから読み込みます。
        /// </summary>
        /// <param name="filepath"></param>
        public bool Make_FromFile(string filepath)
        {
            bool successful = false;

            if (!File.Exists(filepath))
            {
                MessageBox.Show("ファイルがありません。\npath=[" + filepath + "]", "情報");
                goto gt_EndMethod;
            }

#if DEBUG
            MessageBox.Show(
                "filepath=[" + filepath + "]\n"
            , "デバッグ");
#endif

            List<List<string>> table = Util_Csv.ReadCsv(filepath);

            int rowVersion = 0;
            int rowKomawari = 0;
            int rowKk = 0;
            int rowKp = 0;
            int rowPp = 0;

            foreach(List<string> row in table)
            {
                if(row.Count<1)//空行
                {
                    goto gt_Next;
                }

                string col0 = row[0].Trim();

                if (col0.StartsWith("#"))//コメント行
                {
                    goto gt_Next;
                }
                
                switch(col0)
                {
                    case "Version":
                        rowVersion++;
                        break;
                    case "Komawari":

                        if (row.Count < 2)//エラー
                        {
                            rowKomawari++;
                            goto gt_Next;
                        }
                        else
                        {
                            int col2;
                            int.TryParse( row[2], out col2);

                            switch(rowKomawari)
                            {
                                case 0: this.Komawari[(int)Komasyurui14.H01_Fu_____] = col2; break;
                                case 1: this.Komawari[(int)Komasyurui14.H02_Kyo____] = col2; break;
                                case 2: this.Komawari[(int)Komasyurui14.H03_Kei____] = col2; break;
                                case 3: this.Komawari[(int)Komasyurui14.H04_Gin____] = col2; break;
                                case 4: this.Komawari[(int)Komasyurui14.H05_Kin____] = col2; break;
                                case 5: this.Komawari[(int)Komasyurui14.H06_Oh_____] = col2; break;
                                case 6: this.Komawari[(int)Komasyurui14.H07_Hisya__] = col2; break;
                                case 7: this.Komawari[(int)Komasyurui14.H08_Kaku___] = col2; break;
                                case 8: this.Komawari[(int)Komasyurui14.H11_Tokin__] = col2; break;
                                case 9: this.Komawari[(int)Komasyurui14.H12_NariKyo] = col2; break;
                                case 10: this.Komawari[(int)Komasyurui14.H13_NariKei] = col2; break;
                                case 11: this.Komawari[(int)Komasyurui14.H14_NariGin] = col2; break;
                                case 12: this.Komawari[(int)Komasyurui14.H09_Ryu____] = col2; break;
                                case 13: this.Komawari[(int)Komasyurui14.H10_Uma____] = col2; break;
                                default: break;
                            }
                        }
                        rowKomawari++;
                        break;
                    case "KK":
                        if (row.Count < 2+ConstShogi.BAN_SIZE)//エラー
                        {
                            rowKk++;
                            goto gt_Next;
                        }
                        else
                        {
                            for (int k2 = 0; k2 < ConstShogi.BAN_SIZE;k2++ )
                            {
                                int col;
                                int.TryParse(row[2+k2], out col);

                                this.NikomaKankei_KK[rowKk, k2] = col;
                            }
                        }
                        rowKk++;
                        break;
                    case "KP":
                        if (row.Count < 2 + FeatureVectorImpl.CHOSA_KOMOKU_P)//エラー
                        {
                            rowKp++;
                            goto gt_Next;
                        }
                        else
                        {
                            for (int p2 = 0; p2 < FeatureVectorImpl.CHOSA_KOMOKU_P; p2++)
                            {
                                int col;
                                int.TryParse(row[2 + p2], out col);

                                this.NikomaKankei_KP[rowKp, p2] = col;
                            }
                        }
                        rowKp++;
                        break;
                    case "PP":
                        if (row.Count < 2 + FeatureVectorImpl.CHOSA_KOMOKU_P)//エラー
                        {
                            rowPp++;
                            goto gt_Next;
                        }
                        else
                        {
                            for (int p2 = 0; p2 < FeatureVectorImpl.CHOSA_KOMOKU_P; p2++)
                            {
                                int col;
                                int.TryParse(row[2 + p2], out col);

                                this.NikomaKankei_PP[rowPp, p2] = col;
                            }
                        }
                        rowPp++;
                        break;
                    default:
                        break;
                }


            gt_Next:
                ;
            }

#if DEBUG
            MessageBox.Show(
                "rowVersion=[" + rowVersion + "]\n"+
                    "rowKomawari=[" + rowKomawari + "]\n" +
                        "rowKk=[" + rowKk + "]\n" +
                            "rowKp=[" + rowKp + "]\n" +
                                "rowPp=[" + rowPp + "]\n"
            , "デバッグ");
#endif
            successful = true;
        gt_EndMethod:
            return successful;
        }

        /// <summary>
        /// 0クリアーでデータを作成します☆
        /// </summary>
        public void Make_Zero()
        {
            //
            // 駒割は固定。
            // コーディングの利便上、エラー駒やヌル駒にもランダム値を入れておく。
            //
            foreach(Komasyurui14 komaSyurui in Array_Komasyurui.Items_All)//
            {
                this.Komawari[(int)komaSyurui] = 0;
            }
            //
            // 
            //
            this.Komawari[(int)Komasyurui14.H00_Null___] = 0;
            this.Komawari[(int)Komasyurui14.H01_Fu_____] = 0;
            this.Komawari[(int)Komasyurui14.H02_Kyo____] = 0;
            this.Komawari[(int)Komasyurui14.H03_Kei____] = 0;
            this.Komawari[(int)Komasyurui14.H04_Gin____] = 0;
            this.Komawari[(int)Komasyurui14.H05_Kin____] = 0;
            this.Komawari[(int)Komasyurui14.H06_Oh_____] = 0;
            this.Komawari[(int)Komasyurui14.H07_Hisya__] = 0;
            this.Komawari[(int)Komasyurui14.H08_Kaku___] = 0;
            this.Komawari[(int)Komasyurui14.H09_Ryu____] = 0;
            this.Komawari[(int)Komasyurui14.H10_Uma____] = 0;
            this.Komawari[(int)Komasyurui14.H11_Tokin__] = 0;
            this.Komawari[(int)Komasyurui14.H12_NariKyo] = 0;
            this.Komawari[(int)Komasyurui14.H13_NariKei] = 0;
            this.Komawari[(int)Komasyurui14.H14_NariGin] = 0;

            //
            // KK
            //
            for (int iJigyoku = 0; iJigyoku < ConstShogi.BAN_SIZE; iJigyoku++)//自玉
            {
                for (int iAitegyoku = 0; iAitegyoku < ConstShogi.BAN_SIZE; iAitegyoku++)//相手玉
                {
                    this.NikomaKankei_KK[iJigyoku, iAitegyoku] = 0;
                }
            }

            //
            // KP
            //
            for (int iJigyoku = 0; iJigyoku < ConstShogi.BAN_SIZE; iJigyoku++)//自玉
            {
                for (int iChosaKomoku = 0; iChosaKomoku < FeatureVectorImpl.CHOSA_KOMOKU_P; iChosaKomoku++)//調査項目Ｐ
                {
                    this.NikomaKankei_KP[iJigyoku, iChosaKomoku] = 0;
                }
            }

            //
            // PP
            //
            for (int iChosaKomoku1 = 0; iChosaKomoku1 < FeatureVectorImpl.CHOSA_KOMOKU_P; iChosaKomoku1++)//調査項目Ｐ
            {
                for (int iChosaKomoku2 = 0; iChosaKomoku2 < FeatureVectorImpl.CHOSA_KOMOKU_P; iChosaKomoku2++)//調査項目Ｐ
                {
                    this.NikomaKankei_PP[iChosaKomoku1, iChosaKomoku2] = 0;
                }
            }
        }

    }

}
