﻿using Grayscale.P211_WordShogi__.L250____Word;
using Grayscale.P218_Starlight__.L___500_Struct;
using Grayscale.P226_Tree_______.L___500_Struct;
using Grayscale.P247_KyokumenWra.L500____Struct;
using Grayscale.P322_KifuTree___.L___250_Struct;
using Grayscale.P511_KifuWarabe_.L___020_Kokoro;
using System.Collections.Generic;

namespace Grayscale.P511_KifuWarabe_.L080____ScoreSibori
{

    /// <summary>
    /// 棋譜ツリーのスコアの低いノードを捨てていきます。
    /// </summary>
    public class ScoreSiboriEngine
    {

        /// <summary>
        /// 棋譜ツリーの　評価値の高いノードだけを残して、評価値の低いノードをばっさばっさ捨てて行きます。
        /// </summary>
        /// <param name="kifu"></param>
        /// <param name="atamanosumiCollection"></param>
        public void EdaSibori_HighScore(KifuTree kifu, Kokoro atama)
        {
            //
            // ノードが２つもないようなら、スキップします。
            //
            if (kifu.CurNode.Count_ChildNodes < 2)
            {
                goto gt_EndMethod;
            }


            List<Node<Starbeamable, KyokumenWrapper>> rankedNodes = this.RankingNode_WithJudge_ForeachNextNodes(kifu.CurNode);

            Dictionary<string, Node<Starbeamable, KyokumenWrapper>> dic = new Dictionary<string, Node<Starbeamable, KyokumenWrapper>>();
            if (kifu.CurNode.Value.ToKyokumenConst.KaisiPside == Playerside.P1)
            {
                // 1番高いスコアを調べます。
                double goodestScore = double.MinValue;
                foreach (Node<Starbeamable, KyokumenWrapper> node in rankedNodes)
                {
                    if (node is KifuNode)
                    {
                        double score = ((KifuNode)node).KyHyoka.Total();

                        if (goodestScore < score)
                        {
                            goodestScore = score;
                        }
                    }
                }

                // 1番良いスコアのノードだけ残します。
                kifu.CurNode.Foreach_ChildNodes((string key, Node<Starbeamable, KyokumenWrapper> node, ref bool toBreak) =>
                {
                    double score;
                    if (node is KifuNode)
                    {
                        score = ((KifuNode)node).KyHyoka.Total();
                    }
                    else
                    {
                        score = 0.0d;
                    }

                    if (goodestScore <= score)
                    {
                        dic.Add(key, node);
                    }
                });
            }
            else
            {
                // 2Pは、マイナスの方が良い。
                double goodestScore = double.MaxValue;
                foreach (Node<Starbeamable, KyokumenWrapper> node in rankedNodes)
                {
                    if (node is KifuNode)
                    {
                        double score = ((KifuNode)node).KyHyoka.Total();

                        if (score < goodestScore)//より負の値を選びます。
                        {
                            goodestScore = score;
                        }
                    }
                }

                // 1番良いスコアのノードだけ残します。
                kifu.CurNode.Foreach_ChildNodes((string key, Node<Starbeamable, KyokumenWrapper> node, ref bool toBreak) =>
                {
                    double score;
                    if (node is KifuNode)
                    {
                        score = ((KifuNode)node).KyHyoka.Total();
                    }
                    else
                    {
                        score = 0.0d;
                    }

                    if (score <= goodestScore)//より負数の方がよい。
                    {
                        dic.Add(key, node);
                    }
                });
            }





            if (kifu.CurNode.Count_ChildNodes == dic.Count)
            {
                // 全ての手が同じスコアなら、妄想は失敗かもしれません。

                // FIXME: 他の手が原因で、全ての手が同じスコアになってしまった可能性がある☆
                foreach (Tenonagare atamanosumi in atama.TenonagareItems)
                {
                    atamanosumi.ResultKioku.Sippai++;
                }
            }


            // 枝を更新します。
            kifu.CurNode.PutSet_ChildNodes( dic);

        gt_EndMethod:
            ;
        }



        /// <summary>
        /// 局面が、妄想に近いかどうかで点数付けします。
        /// </summary>
        /// <param name="nextNodes"></param>
        /// <returns></returns>
        public List<Node<Starbeamable, KyokumenWrapper>> RankingNode_WithJudge_ForeachNextNodes( Node<Starbeamable, KyokumenWrapper> hubNode)
        {
            // ランク付けしたあと、リスト構造に移し変えます。
            List<Node<Starbeamable, KyokumenWrapper>> list = new List<Node<Starbeamable, KyokumenWrapper>>();

            hubNode.Foreach_ChildNodes((string key, Node<Starbeamable, KyokumenWrapper> node, ref bool toBreak) =>
            {
                list.Add(node);
            });

            // ランク付けするために、リスト構造に変換します。

            ScoreSiboriEngine.Sort(list);

            return list;
        }


        public static void Sort(List<Node<Starbeamable, KyokumenWrapper>> items)
        {
            items.Sort((a, b) =>
            {
                double bScore;
                double aScore;

                if (b is KifuNode)
                {
                    bScore = ((KifuNode)b).KyHyoka.Total();
                }
                else
                {
                    bScore = 0.0d;
                }


                if (a is KifuNode)
                {
                    aScore = ((KifuNode)a).KyHyoka.Total();
                }
                else
                {
                    aScore = 0.0d;
                }

                return (int)(bScore - aScore);//点数が大きいほうが前に行きます。
            });

        }

    }

}
